require 'csv'

module TableauTools

  class Anonymizer

    attr_reader   :anonWords, :low, :high
    attr_accessor :input, :output, :field

    @minWords = 2
    @maxWords = 4

    @@anonymousText = <<ANONTEXT
Lorem ipsum dolor sit amet consectetur adipiscing elit Quisque in dolor quis risus rutrum hendrerit sed et dolor Curabitur 
ac ipsum dapibus bibendum nisi id eleifend nisl Nam sagittis nisl sed massa ultricies eu finibus nibh consectetur 
Suspendisse nec nulla lacinia sagittis eros vitae varius sapien Morbi pulvinar tellus vitae consectetur rhoncus Praesent 
nec consectetur purus vitae pretium felis Fusce non leo at ex lacinia molestie Sed malesuada nisl laoreet nisi consequat 
suscipit at in ligula Etiam bibendum elit a posuere euismod sem urna ultrices ex eget tincidunt enim est ac enim Aliquam 
ac arcu facilisis venenatis urna sit amet euismod nunc Suspendisse eget rhoncus orci Sed odio leo condimentum finibus 
condimentum id ultrices eget odio Maecenas pellentesque ipsum vitae feugiat tincidunt Maecenas ornare eros finibus eros 
consectetur tincidunt Phasellus fringilla orci vitae risus laoreet viverra volutpat massa fringilla Donec mattis ante sed 
eros sagittis accumsan Nulla nec mauris ultrices ex commodo efficitur Sed pellentesque velit ex eget tempus est malesuada 
eu Cras quis nisi nisi Fusce tincidunt scelerisque elementum Donec arcu metus auctor ac efficitur vitae dapibus id ligula 
Integer faucibus semper mi ac viverra Cras aliquam est laoreet libero mattis feugiat Pellentesque at risus vel eros 
fermentum pharetra nec nec justo Donec pretium ornare fermentum Suspendisse turpis dolor mollis ut blandit non porttitor 
vitae dolor Maecenas a mi imperdiet neque consectetur facilisis eu at tortor Phasellus tincidunt lorem nec eros mattis 
volutpat Curabitur iaculis velit vel nulla consectetur non fermentum neque consectetur Cras hendrerit lobortis quam nec 
convallis erat sagittis a Curabitur dictum egestas ultricies Pellentesque habitant morbi tristique senectus et netus et 
malesuada fames ac turpis egestas Vestibulum non eleifend nunc Donec eu ligula quam Sed efficitur ligula quis lacus 
accumsan et viverra urna cursus Donec efficitur augue ut dapibus volutpat tellus turpis pellentesque est vitae hendrerit 
dui ex nec nisl Fusce gravida efficitur justo eget iaculis nisi semper ut Duis ac condimentum lectus Nullam maximus mollis 
urna Phasellus a odio ante Fusce nisi magna fringilla ac nisi eget volutpat tempus augue Aliquam eu mattis ex Vivamus 
vitae maximus arcu id dictum enim Phasellus molestie bibendum massa nec facilisis mauris venenatis non Praesent finibus 
ullamcorper diam eget volutpat Proin luctus quam id luctus porttitor velit tortor cursus nisi vel egestas eros nunc vel 
nibh Phasellus vel magna nibh Curabitur lectus quam efficitur sit amet justo at cursus pretium sapien Vestibulum molestie 
tempor eros in pulvinar felis ornare vitae Duis erat ante eleifend non elit sed consequat condimentum purus Pellentesque 
id egestas enim Praesent mollis purus eu facilisis convallis urna velit lobortis purus a tristique nisi sem vitae ipsum 
Fusce luctus est at feugiat cursus Vestibulum ultrices turpis vel iaculis commodo tortor eros finibus est et elementum 
nibh quam et eros Sed quis posuere est Suspendisse auctor gravida magna vitae consectetur mi mollis vel Curabitur luctus 
lacus at dictum semper sem metus gravida elit eget auctor lacus diam at odio Integer sagittis ultrices erat vitae semper 
Fusce fermentum dictum dui non ornare nunc consequat porttitor Nulla ut congue urna Morbi mollis tristique feugiat Donec 
id erat sagittis rutrum lacus tempor porta elit Nulla eleifend cursus auctor Sed vel maximus ligula Maecenas a tempor nulla
Etiam eu turpis rutrum tristique tellus in pellentesque purus Phasellus mattis lectus id nulla ultricies finibus Donec 
tempus at odio finibus iaculis Mauris eu ex lobortis tincidunt nisl et aliquam enim Nunc eget laoreet nunc Morbi tincidunt 
tortor a sollicitudin dictum ligula felis semper nulla vitae placerat dui justo in nulla Curabitur semper sapien non 
gravida mattis urna tellus congue erat ut vestibulum sem lectus a dolor Aliquam luctus eu tortor nec pulvinar Duis 
vestibulum lacus eu sollicitudin laoreet Nam eu arcu pellentesque diam vehicula euismod at dictum nibh Maecenas lobortis 
justo ac finibus rutrum magna urna consequat nulla sed sodales sapien ante ut purus Phasellus sed commodo ante Maecenas 
eget nisi vel orci placerat pretium id in tellus Orci varius natoque penatibus et magnis dis parturient montes nascetur 
ridiculus mus Phasellus ac lacus vehicula cursus ex euismod dapibus lorem Nam vitae est feugiat condimentum orci id 
blandit nulla Donec non urna vitae eros iaculis varius in at nisl Fusce vulputate placerat bibendum Cras id tellus 
porttitor sagittis sapien vel viverra est In hac habitasse platea dictumst Praesent tellus mi dictum ut finibus eu 
blandit non elit Duis in elit ipsum Vivamus vel elementum quam Curabitur ac felis et urna bibendum mollis eget quis 
ipsum Vivamus sit amet dignissim urna Nam hendrerit condimentum tortor vel auctor Integer suscipit neque non vulputate 
interdum est sem molestie ligula vitae lobortis arcu sapien eu velit Fusce a risus porta malesuada mauris ac tincidunt 
massa Phasellus congue nisi eu tortor egestas ut dignissim nunc consectetur Proin dictum sem orci eget feugiat augue 
tincidunt feugiat Phasellus in massa in purus tincidunt tempus Vivamus vitae magna sed metus auctor malesuada in vitae 
ex Vivamus viverra faucibus nibh sit amet pretium Cras lobortis lectus sit amet magna iaculis semper Curabitur id turpis 
blandit efficitur lacus a luctus augue In at faucibus nulla Phasellus et ligula sapien Class aptent taciti sociosqu ad 
litora torquent per conubia nostra per inceptos himenaeos Nam condimentum lacus vel nunc interdum dignissim Cras 
consectetur augue nec felis maximus nec lacinia nunc tincidunt Praesent blandit lacinia urna vitae egestas Phasellus 
bibendum fermentum ex vitae molestie Mauris placerat erat sit amet elit maximus et rutrum augue sagittis Duis vestibulum 
velit lacus vitae scelerisque tortor fringilla vel Sed a accumsan sem Nulla blandit molestie viverra Sed ut ligula 
vulputate lobortis felis in congue ligula Curabitur faucibus odio dolor nec gravida tellus scelerisque a Mauris lacinia 
accumsan neque Curabitur imperdiet finibus iaculis Quisque scelerisque tempor libero non sodales tellus fermentum ut 
Maecenas ornare sed dolor sit amet condimentum Proin augue erat eleifend eu lacus at facilisis lobortis sapien Morbi 
accumsan quam et egestas varius Nam nec nisl varius mollis dui id ultricies diam Mauris elementum dolor massa dictum 
egestas augue tristique nec In nisl urna aliquam nec commodo sit amet tristique quis elit Aliquam nunc sem convallis 
sit amet felis non feugiat pharetra tortor Nulla facilisi Donec in est et tellus consequat accumsan a cursus erat Quisque 
porttitor dolor ac nibh aliquam eget accumsan ante aliquet Integer sit amet dolor vestibulum feugiat nisi vitae mollis 
urna Proin tempus massa odio eu finibus lectus eleifend in Curabitur quis vehicula elit Sed accumsan nulla risus nec 
dictum lacus gravida vel Sed aliquet mauris nisl non venenatis est ullamcorper maximus Vestibulum id ipsum iaculis 
congue metus eu vehicula nulla Pellentesque vehicula ex at efficitur laoreet Nulla vestibulum condimentum mauris a 
pulvinar sapien porttitor consequat In pulvinar nunc a nibh pretium ac fermentum justo facilisis Aliquam nisl turpis 
fermentum at ipsum ac pharetra fringilla nulla Donec gravida lobortis nulla quis ultricies ipsum porttitor mollis Proin 
a mi varius orci tristique auctor in vel lectus Pellentesque hendrerit venenatis eros vitae tincidunt justo congue in 
Morbi non imperdiet quam a auctor lacus Class aptent taciti sociosqu ad litora torquent per conubia nostra per inceptos 
himenaeos Morbi diam ipsum rhoncus nec rhoncus ac venenatis et nunc Donec magna tellus ultricies sit amet libero vitae 
faucibus consectetur nulla Fusce tincidunt in massa vel sollicitudin Sed tellus lorem ullamcorper non lacus rutrum 
semper luctus magna Ut eget sollicitudin tortor Aenean vel laoreet metus sed scelerisque velit Nunc molestie metus 
posuere eleifend sagittis turpis dui feugiat nisi ac tincidunt nunc nulla ac mi Proin dictum diam felis quis blandit 
magna tempor et Aenean luctus sem arcu at scelerisque ex venenatis ac Cras venenatis elementum posuere Etiam dictum 
maximus blandit Vestibulum nunc turpis lacinia ut viverra in fringilla ut massa Aenean sed nulla id nulla consectetur 
dictum a non velit Aliquam convallis lectus auctor mollis risus id mollis sapien Cras eleifend justo risus a vulputate 
neque lobortis quis Suspendisse consequat nulla arcu quis pretium nisi dignissim quis Nunc convallis dictum consequat 
Curabitur et egestas est Ut scelerisque elit a nisl condimentum fermentum Proin tempus est non leo varius imperdiet Morbi 
eget pellentesque odio Pellentesque tristique ultricies lacus Ut id viverra leo Morbi condimentum augue non ligula 
bibendum faucibus Proin tempor ex non laoreet dapibus erat dui lacinia tortor dapibus malesuada eros quam lacinia leo 
Quisque lobortis dui id ultricies vestibulum Integer sodales placerat enim vitae luctus nibh cursus vel In sit amet orci 
eros Vivamus vulputate faucibus dolor nec mollis tortor vestibulum in Fusce congue dictum lacus ut pharetra Quisque quis 
tristique libero In sollicitudin sagittis justo Sed cursus velit neque Proin eu turpis sed massa tristique consequat Nunc 
finibus nibh quis cursus sagittis Proin nisl nisl pulvinar eget molestie id tristique sed justo Morbi ligula ligula 
tristique nec diam nec viverra elementum nibh Vivamus eros lectus consectetur sollicitudin varius vitae tincidunt vel 
neque Aliquam dignissim egestas dui vel faucibus purus mattis tristique Proin et placerat lacus Vestibulum consectetur 
elit vitae mollis iaculis Aenean dictum nec nunc nec dapibus Phasellus cursus dictum purus eget consectetur nisi sodales 
sit amet Vestibulum sed nisl magna Phasellus ut tellus et nisl aliquam faucibus Morbi dictum risus sit amet nulla 
facilisis quis feugiat eros suscipit Suspendisse eleifend justo ac vulputate feugiat orci dui lacinia dolor in lobortis 
tellus erat et erat Nulla facilisi Ut quis lectus magna Nullam in elementum nulla a placerat diam Etiam suscipit 
dignissim vulputate Duis sit amet diam sit amet urna condimentum eleifend quis non ipsum Etiam maximus erat sed dapibus 
vehicula Maecenas feugiat ipsum et lectus aliquet varius eu eu libero Curabitur sit amet dui fringilla sagittis tellus 
sit amet tincidunt elit In aliquam non odio ut sodales Pellentesque vel tellus eget elit dictum sollicitudin vel in mi 
Aliquam varius purus vitae aliquam aliquam Morbi lectus lorem ullamcorper a sollicitudin in dignissim a erat Nam 
sollicitudin massa in mi laoreet euismod Sed eget dui porttitor pretium lectus in iaculis mauris Fusce congue velit 
felis id ultrices sapien vulputate vel Aenean semper arcu leo ut feugiat quam interdum a Donec et metus nisl 
Pellentesque consequat posuere massa eu ornare libero bibendum quis Integer consectetur dolor leo in aliquet dui 
consectetur in Aliquam erat volutpat Quisque molestie aliquam enim ac vulputate turpis ullamcorper nec Proin commodo 
orci quis libero eleifend pellentesque scelerisque ante fermentum Donec ac ligula sed nisl feugiat commodo vitae et 
tellus Interdum et malesuada fames ac ante ipsum primis in faucibus In commodo posuere dui sit amet bibendum Phasellus 
vestibulum dignissim massa id feugiat Donec pretium malesuada orci id fermentum sem ultricies nec Etiam justo libero 
bibendum sit amet blandit id tincidunt at magna Sed malesuada sem ut lacus dictum condimentum quis quis eros Duis a 
nulla sit amet libero blandit volutpat eu sit amet dolor Morbi in ullamcorper purus Phasellus justo enim mollis a enim 
sit amet blandit imperdiet tellus Morbi euismod diam non lorem tincidunt volutpat Cras ultrices euismod vulputate 
Suspendisse fermentum velit eu vehicula vestibulum orci quam hendrerit ligula ac venenatis eros magna a velit Mauris 
maximus blandit lacinia Orci varius natoque penatibus et magnis dis parturient montes nascetur ridiculus mus Nulla 
convallis et massa a cursus Aenean gravida est eros id commodo nunc scelerisque nec Sed vitae metus rhoncus rhoncus 
sapien sit amet auctor magna Maecenas tincidunt nunc ac lacus luctus sit amet ultricies leo eleifend Nulla semper enim 
ipsum sit amet facilisis risus malesuada sed Pellentesque suscipit eros erat vitae hendrerit nisi aliquet vitae Ut libero 
metus imperdiet vitae ante eget tempor tristique velit Nullam nunc libero convallis id purus sit amet pellentesque 
tristique orci Nulla elementum dui ut aliquam vulputate augue lacus eleifend urna nec eleifend massa urna at eros In 
mattis lorem sit amet consectetur luctus nisi lectus scelerisque magna at maximus lorem lorem sed odio Nullam at 
molestie mi Cras eget accumsan quam Vivamus consequat dignissim lacus non lobortis Nam quis odio nisi Donec commodo 
leo ac hendrerit faucibus lacus neque hendrerit ante quis bibendum mauris erat fringilla tortor Nulla cursus consectetur 
enim quis maximus Duis sit amet neque sed urna tincidunt posuere eu sed turpis Mauris mollis purus justo et sagittis 
tellus scelerisque eu Nulla et mauris eget neque efficitur egestas Sed tincidunt tellus sit amet felis venenatis at 
scelerisque ex rutrum Integer auctor quam a aliquam tincidunt purus lacus feugiat risus id volutpat felis tellus sed 
tellus Donec posuere tortor ut metus suscipit finibus Praesent vehicula a justo at egestas Nullam quis dolor quis mi 
fringilla egestas sodales tincidunt lorem Sed porta eros tortor at sodales nisl scelerisque id Ut condimentum malesuada 
euismod Proin gravida luctus tincidunt Vivamus id velit suscipit maximus leo nec dapibus risus Vivamus sed blandit sem 
Morbi maximus magna efficitur fringilla luctus nisi lacus laoreet nisl id interdum magna nisi quis nulla Sed id massa 
et ipsum venenatis commodo Ut interdum accumsan sollicitudin Fusce interdum orci vel posuere commodo Sed pharetra luctus 
metus sed consequat tortor tincidunt vel Vivamus euismod sagittis nisl vitae rutrum Maecenas ante metus tincidunt id 
posuere et euismod a dui Quisque nibh quam pellentesque ac dapibus quis pharetra ac odio Etiam volutpat scelerisque 
convallis Duis in est vel magna ornare vehicula id eget felis Nullam nec commodo mauris sit amet condimentum orci Duis 
a varius risus Ut fermentum laoreet nibh eget viverra Maecenas imperdiet dapibus lacinia Nullam faucibus turpis 
vestibulum magna mollis aliquet ultrices odio mollis Nulla hendrerit nec velit sit amet convallis Maecenas a massa 
mauris Vestibulum imperdiet volutpat lobortis In a dictum tortor Proin vel dui a turpis lacinia lobortis Maecenas eget 
pellentesque ex non mattis nibh Ut suscipit dapibus tellus id ornare nisl aliquam quis Aliquam sit amet sapien purus 
Nam finibus urna eget dignissim euismod risus dui accumsan eros vel laoreet dui leo nec enim Proin sit amet ligula 
sollicitudin volutpat leo et euismod nulla Donec iaculis ligula feugiat purus imperdiet congue sit amet a lacus 
Suspendisse vitae odio gravida tincidunt lorem at lacinia metus Suspendisse id ex in elit feugiat ultricies Sed 
sollicitudin bibendum nunc Nam ut ultrices velit Suspendisse a vestibulum sem Duis iaculis tellus vitae rutrum pharetra 
Curabitur in nisi lobortis rhoncus tellus porta elementum odio Donec mauris dolor vestibulum congue est ac posuere 
consequat ligula Proin eu est maximus consectetur elit at aliquet ipsum Aenean id vehicula est Ut tincidunt odio eget 
pretium accumsan nisi justo interdum lectus vitae aliquet tortor arcu ac eros Nam commodo enim at leo ultricies varius 
Donec sollicitudin magna ut est vulputate et bibendum nisl congue Nam blandit sit amet purus ut vehicula Mauris feugiat 
quam a sodales sodales mauris libero porta purus quis viverra lectus purus nec neque Proin mattis tempor tincidunt 
Nullam ornare a velit vel rutrum Sed ullamcorper a arcu vel posuere Vivamus malesuada ornare arcu eget ullamcorper 
Pellentesque in metus sit amet augue sollicitudin volutpat vitae id lectus Proin in vestibulum orci Proin in nibh mauris 
Praesent sodales feugiat dapibus In vel pharetra quam in mollis lectus Mauris quis sollicitudin dui Vestibulum malesuada 
enim et tincidunt tempor arcu nisi ornare nibh et venenatis nulla magna ac nibh Phasellus tempus tellus dolor at euismod 
nunc pharetra eu Donec id ornare diam Donec at lacinia dolor Nulla eu malesuada turpis Morbi finibus sem semper feugiat 
dapibus Sed vel eleifend enim aliquet gravida neque Donec pellentesque aliquet turpis ac facilisis Morbi sit amet dui 
fermentum tristique diam at dignissim velit Vivamus ac varius urna Aliquam erat volutpat Nunc condimentum sed lacus eu 
tincidunt In massa nunc aliquam sit amet urna ac luctus ultricies eros In rutrum mauris vitae ante lacinia ut pharetra 
magna lobortis Etiam a nibh id nulla rutrum finibus Class aptent taciti sociosqu ad litora torquent per conubia nostra 
per inceptos himenaeos Suspendisse commodo facilisis urna et pharetra Phasellus nisl diam interdum et diam vel congue 
fermentum velit Fusce urna risus mattis nec mollis vitae auctor et augue Donec aliquet ante eu dolor lacinia eget 
elementum turpis varius Fusce quis turpis ex Mauris eget diam mattis purus porttitor dapibus et eget tellus Nulla feugiat 
a diam quis tincidunt Class aptent taciti sociosqu ad litora torquent per conubia nostra per inceptos himenaeos Aenean ac 
ex dapibus ultrices metus vitae bibendum nunc Nullam at sollicitudin augue vitae pulvinar nulla Nullam ac volutpat massa 
at venenatis ipsum Praesent suscipit euismod est id semper justo ullamcorper a In et cursus purus Nunc ut augue a mauris 
imperdiet pellentesque Nam aliquam convallis ipsum vitae consectetur Etiam ut convallis odio Duis vitae laoreet risus 
eget aliquet elit Suspendisse semper nibh velit vel blandit ligula consequat non Donec a dictum justo Vivamus dignissim 
turpis vel eros tincidunt pretium Morbi consectetur erat et leo ultrices suscipit Maecenas venenatis libero in purus 
tincidunt in venenatis tortor faucibus Suspendisse ullamcorper nibh lacus Vestibulum eget est sodales velit mattis tempus 
Sed egestas laoreet lectus ac luctus Donec interdum purus justo eget lobortis risus iaculis at In turpis enim aliquam sed 
metus sit amet luctus feugiat diam Suspendisse feugiat mollis accumsan Ut vitae lorem sed nunc sollicitudin ultrices 
imperdiet aliquam arcu Class aptent taciti sociosqu ad litora torquent per conubia nostra per inceptos himenaeos Morbi ut 
nulla ut purus laoreet faucibus In vitae maximus orci Aenean mollis metus ac placerat ullamcorper augue ante viverra leo 
a lobortis magna nisi ac neque Interdum et malesuada fames ac ante ipsum primis in faucibus Integer congue libero urna at 
elementum risus porta id Vivamus maximus porttitor rhoncus Vivamus vulputate mauris id neque ornare non sagittis lectus 
euismod Aenean facilisis tellus eget erat consectetur id dictum erat placerat Vestibulum tempus dapibus ante eget iaculis 
Vestibulum ac arcu nisi Morbi sed nulla sollicitudin pellentesque justo sit amet commodo turpis Morbi mauris mauris 
facilisis eget molestie vel imperdiet in mi Nunc a semper libero vel lacinia nulla Nullam rhoncus viverra nisi varius 
scelerisque neque ullamcorper ut Pellentesque cursus velit ac leo laoreet quis vehicula odio pretium Aenean lacinia urna 
in magna egestas facilisis laoreet dolor vehicula In in rutrum mauris Mauris elementum orci nec orci eleifend at 
pellentesque felis porttitor Nulla pretium consequat diam vel gravida Suspendisse posuere volutpat lacus vel finibus 
Integer libero odio tempor eu nulla ut blandit blandit risus Donec et mauris nec orci condimentum porta eu vitae risus 
Aliquam cursus sit amet dolor eget eleifend Cras et eros ut elit malesuada pulvinar Aenean laoreet elit at mauris eleifend 
eget aliquam
ANONTEXT

    @codedWords = {}

    def initialize
      @anonWords = @@anonymousText.gsub(/\n/,' ').split
      @coded = {}
      @low   = 0
      @limit = @anonWords.length-1
      @rand  = Random.new
      @minWords  = 2 
      @maxWords  = 4
      @anonCSV   = CSV.open('Anonymized.csv','w')
      @anonCSV  << ['File - original','File - anon','Record #','Field','Value - original','Value - anon']
    end

    def anonymize(input, fields,  output)
      puts "Anonymizing %-25s  to  %-25s  - %s " % [input, output, fields.inspect]
      raise 'The input file must be specified' if     input.nil?
      raise 'The input file must exist'        unless File.exist?(input)
      raise 'The field(s) to anonymize must be provided as a String or an array of Strings' unless fields.instance_of?(String) || fields.instance_of?(Array)
      @input    = input
      @output   = output
      @anonVals = {}
      csvFields = CSV.open(@input, &:readline) 
      output    = CSV.open(@output,'w')
      output << csvFields
      fields = [fields] if fields.is_a? String
      recNum = 0
      CSV.foreach(@input, :headers => true) do |row|
        fields.each do |field|
          unless row[field].nil?
            init = row[field]
            anon = anonWords(row[field])
            row[field] = anon
            @anonCSV << [@input,@output,recNum+=1,field,init,anon]
          end
        end
        output << row
      end
    end

    def anonWords value
      puts "anonymizing:  %25s  :: %-6s  ->  %-s " % [value, @anonVals[value].nil?, @anonVals[value]]  
      return @anonVals[value] unless @anonVals[value].nil?
      range = rand(@minWords..@maxWords)
      limit = @limit - range 
      first = rand(range..limit)
      last  = first + range - 1
      words = @anonWords[first..last].join(' ')
      @anonVals[value] = words
    end

    def codeValue value
      if @codedWords[value].nil?
        @codedWords[value] = anonWords
      end
      @codedWords[value]
    end


  end # class Anonymizer

end # module utility